==> json-joy-5.0.0.tgz: package/esm/reactive-rpc/common/rpc/RpcApiCaller.js <==
import { EMPTY, firstValueFrom, from, interval, Observable, of, Subject } from 'rxjs';
import { catchError, debounce, finalize, first, map, mergeWith, share, switchMap, take, tap } from 'rxjs/operators';
import { RpcServerError } from './constants';
import { BufferSubject } from '../../../util/BufferSubject';
import { ErrorLikeErrorFormatter, RpcError } from './error';
export class RpcApiCaller {
    constructor({ api, error, preCallBufferSize = 10, maxActiveCalls = 50 }) {
        this._calls = 0;
        this.api = api;
        this.error = error || new ErrorLikeErrorFormatter();
        this.preCallBufferSize = preCallBufferSize;
        this.maxActiveCalls = maxActiveCalls;
    }
    get activeCalls() {
        return this._calls;
    }
    exists(name) {
        return this.api.hasOwnProperty(name);
    }
    get(name) {
        if (!this.exists(name))
            throw new RpcError(RpcServerError.NoMethodSpecified);
        return this.api[name];
    }
    async call(name, request, ctx) {
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new RpcError(RpcServerError.TooManyActiveCalls);
            const method = this.get(name);
            if (method.validate)
                method.validate(request);
            try {
                this._calls++;
                if (method.onPreCall)
                    await method.onPreCall(ctx, request);
                return !method.isStreaming
                    ? await method.call(ctx, request)
                    : await firstValueFrom(method.call$(ctx, of(request)));
            }
            finally {
                this._calls--;
            }
        }
        catch (error) {
            throw this.error.format(error);
        }
    }
    createCall(name, ctx) {
        const req$ = new Subject();
        const reqUnsubscribe$ = new Subject();
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new RpcError(RpcServerError.TooManyActiveCalls);
            const method = this.get(name);
            if (!method.isStreaming) {
                const response$ = from((async () => {
                    const request = await firstValueFrom(req$.pipe(first()));
                    const response = await this.call(name, request, ctx);
                    return response;
                })());
                const res$ = new Subject();
                response$.subscribe(res$);
                const $resWithErrorsFormatted = res$.pipe(catchError((error) => {
                    throw this.error.format(error);
                }));
                return { req$, reqUnsubscribe$, res$: $resWithErrorsFormatted };
            }
            const methodStreaming = method;
            const requestValidated$ = req$.pipe(map((request) => {
                if (methodStreaming.validate)
                    methodStreaming.validate(request);
                return request;
            }));
            const bufferSize = methodStreaming.preCallBufferSize || this.preCallBufferSize;
            const requestBuffered$ = new BufferSubject(bufferSize);
            requestBuffered$.subscribe({ error: () => { } });
            const error$ = new Subject();
            requestBuffered$.subscribe({
                error: (error) => {
                    error$.error(error);
                },
            });
            requestValidated$.subscribe(requestBuffered$);
            const result$ = requestBuffered$.pipe(take(1), switchMap((request) => {
                return methodStreaming.onPreCall ? from(methodStreaming.onPreCall(ctx, request)) : from([0]);
            }), switchMap(() => {
                Promise.resolve().then(() => {
                    requestBuffered$.flush();
                });
                return methodStreaming.call$(ctx, requestBuffered$).pipe(finalize(() => {
                    error$.complete();
                }));
            }), share(), mergeWith(error$));
            const resultWithActiveCallTracking$ = of(null).pipe(tap(() => {
                this._calls++;
            }), switchMap(() => result$), finalize(() => {
                error$.complete();
                this._calls--;
            }));
            const res$ = new Observable((observer) => {
                const subscription = resultWithActiveCallTracking$.subscribe(observer);
                const timeout = methodStreaming.timeout ?? 15_000;
                const timeoutSubscription = requestBuffered$
                    .pipe(catchError(() => EMPTY), mergeWith(result$.pipe(catchError(() => EMPTY))), debounce(() => interval(timeout)))
                    .subscribe(() => {
                    const error = new RpcError(RpcServerError.Timeout);
                    error$.error(error);
                });
                return () => {
                    timeoutSubscription.unsubscribe();
                    subscription.unsubscribe();
                };
            });
            const $resWithErrorsFormatted = res$.pipe(catchError((error) => {
                throw this.error.format(error);
            }));
            return { req$, reqUnsubscribe$, res$: $resWithErrorsFormatted };
        }
        catch (error) {
            const errorFormatted = this.error.format(error);
            req$.error(errorFormatted);
            const res$ = new Subject();
            res$.error(errorFormatted);
            return { req$, reqUnsubscribe$, res$ };
        }
    }
    call$(name, request$, ctx) {
        const call = this.createCall(name, ctx);
        request$.subscribe(call.req$);
        return call.res$;
    }
}

==> json-joy-7.11.0.tgz: package/esm/reactive-rpc/common/rpc/RpcApiCaller.js <==
import { firstValueFrom, from, of, Subject } from 'rxjs';
import { catchError, finalize, first, map, mergeWith, share, switchMap, take, tap } from 'rxjs/operators';
import { RpcServerError } from './constants';
import { BufferSubject } from '../../../util/BufferSubject';
import { ErrorLikeErrorFormatter, RpcError } from './error';
import { wrapMethod } from './methods';
export class RpcApiCaller {
    api;
    error;
    preCallBufferSize;
    maxActiveCalls;
    _calls = 0;
    constructor({ types, api, error, preCallBufferSize = 10, maxActiveCalls = 50 }) {
        this.api = {};
        for (const key in api)
            this.api[key] = wrapMethod(api[key], types);
        this.error = error || new ErrorLikeErrorFormatter();
        this.preCallBufferSize = preCallBufferSize;
        this.maxActiveCalls = maxActiveCalls;
    }
    get activeCalls() {
        return this._calls;
    }
    exists(name) {
        return this.api.hasOwnProperty(name);
    }
    get(name) {
        if (!this.exists(name))
            throw new RpcError(RpcServerError.NoMethodSpecified);
        return this.api[name];
    }
    async call(name, request, ctx) {
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new RpcError(RpcServerError.TooManyActiveCalls);
            const method = this.get(name);
            if (method.validate)
                method.validate(request);
            try {
                this._calls++;
                if (method.onPreCall)
                    await method.onPreCall(ctx, request);
                return await method.call(ctx, request);
            }
            finally {
                this._calls--;
            }
        }
        catch (error) {
            throw this.error.format(error);
        }
    }
    async callJson(name, request, ctx) {
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new RpcError(RpcServerError.TooManyActiveCalls);
            const method = this.get(name);
            if (method.validate)
                method.validate(request);
            try {
                this._calls++;
                if (method.onPreCall)
                    await method.onPreCall(ctx, request);
                return await method.callJson(ctx, request);
            }
            finally {
                this._calls--;
            }
        }
        catch (error) {
            throw this.error.format(error);
        }
    }
    async callMsgPack(name, request, ctx) {
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new RpcError(RpcServerError.TooManyActiveCalls);
            const method = this.get(name);
            if (method.validate)
                method.validate(request);
            try {
                this._calls++;
                if (method.onPreCall)
                    await method.onPreCall(ctx, request);
                return await method.callMsgPack(ctx, request);
            }
            finally {
                this._calls--;
            }
        }
        catch (error) {
            throw this.error.format(error);
        }
    }
    createCall(name, ctx, callMethod, callMethod$) {
        const req$ = new Subject();
        const reqUnsubscribe$ = new Subject();
        try {
            if (this._calls >= this.maxActiveCalls)
                throw new RpcError(RpcServerError.TooManyActiveCalls);
            const method = this.get(name);
            if (!method.isStreaming) {
                const response$ = from((async () => {
                    const request = await firstValueFrom(req$.pipe(first()));
                    const response = await callMethod(name, request, ctx);
                    return response;
                })());
                const res$ = new Subject();
                response$.subscribe(res$);
                const $resWithErrorsFormatted = res$.pipe(catchError((error) => {
                    throw this.error.format(error);
                }));
                return { req$, reqUnsubscribe$, res$: $resWithErrorsFormatted };
            }
            const methodStreaming = method;
            const requestValidated$ = req$.pipe(map((request) => {
                if (methodStreaming.validate)
                    methodStreaming.validate(request);
                return request;
            }));
            const bufferSize = methodStreaming.preCallBufferSize || this.preCallBufferSize;
            const requestBuffered$ = new BufferSubject(bufferSize);
            const error$ = new Subject();
            requestBuffered$.subscribe({
                error: (error) => {
                    error$.error(error);
                },
            });
            requestValidated$.subscribe(requestBuffered$);
            const result$ = requestBuffered$.pipe(take(1), switchMap((request) => {
                return methodStreaming.onPreCall ? from(methodStreaming.onPreCall(ctx, request)) : from([0]);
            }), switchMap(() => {
                Promise.resolve().then(() => {
                    requestBuffered$.flush();
                });
                return callMethod$(methodStreaming, ctx, requestBuffered$).pipe(finalize(() => {
                    error$.complete();
                }));
            }), share(), mergeWith(error$));
            const resultWithActiveCallTracking$ = of(null).pipe(tap(() => {
                this._calls++;
            }), switchMap(() => result$), finalize(() => {
                error$.complete();
                this._calls--;
            }));
            const $resWithErrorsFormatted = resultWithActiveCallTracking$.pipe(catchError((error) => {
                throw this.error.format(error);
            }));
            return { req$, reqUnsubscribe$, res$: $resWithErrorsFormatted };
        }
        catch (error) {
            const errorFormatted = this.error.format(error);
            req$.error(errorFormatted);
            const res$ = new Subject();
            res$.error(errorFormatted);
            return { req$, reqUnsubscribe$, res$ };
        }
    }
    call$(name, request$, ctx) {
        const call = this.createCall(name, ctx, (name, req, ctx) => this.call(name, req, ctx), (method, ctx, req$) => method.call$(ctx, req$));
        request$.subscribe(call.req$);
        return call.res$;
    }
    callJson$(name, request$, ctx) {
        const call = this.createCall(name, ctx, (name, req, ctx) => this.callJson(name, req, ctx), (method, ctx, req$) => method.callJson$(ctx, req$));
        request$.subscribe(call.req$);
        return call.res$;
    }
    callMsgPack$(name, request$, ctx) {
        const call = this.createCall(name, ctx, (name, req, ctx) => this.callMsgPack(name, req, ctx), (method, ctx, req$) => method.callMsgPack$(ctx, req$));
        request$.subscribe(call.req$);
        return call.res$;
    }
}
